/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#if !defined(AFX_MBTRANSFORMCOEFFS_H__B5704512_C23C_497A_A794_36691A0D01BE__INCLUDED_)
#define AFX_MBTRANSFORMCOEFFS_H__B5704512_C23C_497A_A794_36691A0D01BE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

H264AVC_NAMESPACE_BEGIN

class YuvMbBuffer;

class H264AVCCOMMONLIB_API MbTransformCoeffs
{
public:
  ErrVal copyPredictionFrom( YuvMbBuffer &rcPred );
  ErrVal copyPredictionTo( YuvMbBuffer &rcPred );
  ErrVal clearPrediction();

  MbTransformCoeffs() ;
  TCoeff*       get   ( LumaIdx   cLumaIdx )          { return &m_aaiLevel[cLumaIdx     ][0]; }
  const TCoeff* get   ( LumaIdx   cLumaIdx )    const { return &m_aaiLevel[cLumaIdx     ][0]; }

  TCoeff*       get   ( ChromaIdx cChromaIdx )        { return &m_aaiLevel[16+cChromaIdx][0]; }
  const TCoeff* get   ( ChromaIdx cChromaIdx )  const { return &m_aaiLevel[16+cChromaIdx][0]; }

  TCoeff*       get8x8( B8x8Idx   c8x8Idx )           { return &m_aaiLevel[4*c8x8Idx.b8x8Index()][0]; }
  const TCoeff* get8x8( B8x8Idx   c8x8Idx )     const { return &m_aaiLevel[4*c8x8Idx.b8x8Index()][0]; }

  TCoeff*       getTCoeffBuffer()                     { return &m_aaiLevel[0][0]; }
  const TCoeff* getTCoeffBuffer()               const { return &m_aaiLevel[0][0]; }

  UInt calcCoeffCount( LumaIdx cLumaIdx, Bool bIs8x8, Bool bInterlaced, UInt uiStart, UInt uiStop ) const;
  UInt calcCoeffCount( ChromaIdx cChromaIdx, const UChar *pucScan, UInt uiStart, UInt uiStop ) const;
  UInt getCoeffCount( LumaIdx cLumaIdx )                   const { return m_aaucCoeffCount[cLumaIdx]; }
  UInt getCoeffCount( ChromaIdx cChromaIdx )               const { return m_aaucCoeffCount[16+cChromaIdx]; }
  Void setCoeffCount( LumaIdx cLumaIdx, UInt uiCoeffCount )      { m_aaucCoeffCount[cLumaIdx] = uiCoeffCount; }
  Void setCoeffCount( ChromaIdx cChromaIdx, UInt uiCoeffCount )  { m_aaucCoeffCount[16+cChromaIdx] = uiCoeffCount; }
  Void clear();
  Void setAllCoeffCount( UChar ucCoeffCountValue = 0 );
  Void copyFrom( const MbTransformCoeffs& rcMbTransformCoeffs );
  Void copyFrom( MbTransformCoeffs& rcMbTransformCoeffs, ChromaIdx cChromaIdx );
  Void copyCoeffCounts( const MbTransformCoeffs& rcMbTransformCoeffs );
  Void reset() {}

	Void dump ( FILE* hFile ) const;

  ErrVal  load( FILE* pFile );
  ErrVal  save( FILE* pFile );

  Void  clearAcBlk                 ( ChromaIdx cChromaIdx );
  Void  clearLumaLevels            ();
  Void  clearChromaLevels          ();
  Void  clearLumaLevels4x4         ( LumaIdx c4x4Idx );
  Void  clearLumaLevels8x8         ( B8x8Idx c8x8Idx );
  Void  clearLumaLevels8x8Block    ( B8x8Idx c8x8Idx );
  Void  clearNewLumaLevels         ( MbTransformCoeffs& rcBaseMbTCoeffs );
  Void  clearNewLumaLevels8x8      ( B8x8Idx c8x8Idx, MbTransformCoeffs& rcBaseMbTCoeffs );
  Void  clearNewLumaLevels8x8Block ( B8x8Idx c8x8Idx, MbTransformCoeffs& rcBaseMbTCoeffs );

  Void storeLevelData              ();
  Void switchLevelCoeffData        ();
  Void add                         ( MbTransformCoeffs* pcCoeffs, Bool bLuma = true, Bool bChroma = true );

protected:
  TCoeff m_aaiLevel[24][16];
  UChar  m_aaucCoeffCount[24];
};



H264AVC_NAMESPACE_END


#endif // !defined(AFX_MBTRANSFORMCOEFFS_H__B5704512_C23C_497A_A794_36691A0D01BE__INCLUDED_)
