/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#if !defined(AFX_MOTIONVECTORCALCULATION_H__9F13FBF7_8AD0_49DA_985B_08EE7CE0F231__INCLUDED_)
#define AFX_MOTIONVECTORCALCULATION_H__9F13FBF7_8AD0_49DA_985B_08EE7CE0F231__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


H264AVC_NAMESPACE_BEGIN


class H264AVCCOMMONLIB_API MotionVectorCalculation
{
protected:
  MotionVectorCalculation();
  virtual ~MotionVectorCalculation();

public:
  static ErrVal create  ( MotionVectorCalculation*& rpcMotionVectorCalculation );
  ErrVal        destroy ();

  ErrVal initSlice( const SliceHeader& rcSH );
  ErrVal uninit();

  ErrVal  calcMvMb    ( MbDataAccess& rcMbDataAccess, MbDataAccess* pcMbDataAccessBase );
  ErrVal  calcMvSubMb ( B8x8Idx c8x8Idx, MbDataAccess& rcMbDataAccess, MbDataAccess* pcMbDataAccessBase );

protected:
  Void xCalc16x16( MbDataAccess& rcMbDataAccess, MbDataAccess* pcMbDataAccessBase );
  Void xCalc16x8( MbDataAccess& rcMbDataAccess, MbDataAccess* pcMbDataAccessBase );
  Void xCalc8x16( MbDataAccess& rcMbDataAccess, MbDataAccess* pcMbDataAccessBase );
  Void xCalc8x8( MbDataAccess& rcMbDataAccess, MbDataAccess* pcMbDataAccessBase, Bool bFaultTolerant );
  Void xCalc8x8( B8x8Idx c8x8Idx, MbDataAccess& rcMbDataAccess, MbDataAccess* pcMbDataAccessBase, Bool bFaultTolerant );

protected:
  UInt m_uiMaxBw;
  Bool m_bSpatialDirectMode;
  Mv3D m_cMvA;
  Mv3D m_cMvB;
  Mv3D m_cMvC;
};


H264AVC_NAMESPACE_END

#endif // !defined(AFX_MOTIONVECTORCALCULATION_H__9F13FBF7_8AD0_49DA_985B_08EE7CE0F231__INCLUDED_)
