/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#if !defined(AFX_PARAMETERSETMNG_H__5F7FDE27_D5CF_4594_A15B_EE376A1650D7__INCLUDED_)
#define AFX_PARAMETERSETMNG_H__5F7FDE27_D5CF_4594_A15B_EE376A1650D7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "H264AVCCommonLib/SequenceParameterSet.h"

#include "list"

#if defined( WIN32 )
# pragma warning( disable: 4251 )
#endif


H264AVC_NAMESPACE_BEGIN

class H264AVCCOMMONLIB_API ParameterSetMng
{
protected:
  ParameterSetMng();
  virtual ~ParameterSetMng() {}

public:
  static ErrVal create( ParameterSetMng*& rpcParameterSetMng );
  ErrVal destroy();
  ErrVal init()                                        { return Err::m_nOK; }
  ErrVal uninit();

  ErrVal setParamterSetList( std::list<SequenceParameterSet*>& rcSPSList, std::list<PictureParameterSet*>& rcPPSList) const;
  Bool   isValidSPS( UInt uiSPSId, Bool bSubSetSPS )   { return m_cSPSBuf.isValidOffset(uiSPSId+(bSubSetSPS?32:0)) && NULL != m_cSPSBuf.get( uiSPSId+(bSubSetSPS?32:0)); }
  Bool   isValidPPS( UInt uiPPSId )                    { return m_cPPSBuf.isValidOffset(uiPPSId) && NULL != m_cPPSBuf.get( uiPPSId); }
  ErrVal get( SequenceParameterSet *& rpcSPS, UInt uiSPSId, Bool bSubSetSPS );
  ErrVal get( PictureParameterSet *& rpcPPS, UInt uiPPSId );
  ErrVal store( SequenceParameterSet* pcSPS );
  ErrVal store( PictureParameterSet* pcPPS );
  Void   setActiveSPS   ( UInt uiSPSId, UInt uiDQId )  { m_auiActiveSPSId[uiDQId] = uiSPSId + ( uiDQId ? NUM_SPS_IDS : 0 ); }
  ErrVal getActiveSPS   ( SequenceParameterSet*& rpcSPS, UInt uiDQId );
  ErrVal getActiveSPSDQ0( SequenceParameterSet*& rpcSPS );

private:
  ErrVal xDestroyPPS(UInt uiPPSId);
  ErrVal xDestroySPS(UInt uiSPSId);

private:
  StatBuf<SequenceParameterSet*,2*NUM_SPS_IDS>  m_cSPSBuf;
  StatBuf<PictureParameterSet*,256> m_cPPSBuf;
  UInt m_auiActiveSPSId[16*8];
  std::list<SequenceParameterSet*>  m_cSPSList;
  std::list<PictureParameterSet*>   m_cPPSList;
};



H264AVC_NAMESPACE_END


#if defined( WIN32 )
# pragma warning( default: 4251 )
#endif

#endif // !defined(AFX_PARAMETERSETMNG_H__5F7FDE27_D5CF_4594_A15B_EE376A1650D7__INCLUDED_)
