/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#if !defined(AFX_SAMPLEWEIGHTING_H__6B0A73D2_EAF3_497F_B114_913D68E1C1C0__INCLUDED_)
#define AFX_SAMPLEWEIGHTING_H__6B0A73D2_EAF3_497F_B114_913D68E1C1C0__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include  "H264AVCCommonLib/YuvMbBuffer.h"

H264AVC_NAMESPACE_BEGIN

class H264AVCCOMMONLIB_API SampleWeighting
{
  typedef Void (*MixSampleFunc) ( Pel* pucDest,  Int iDestStride, Pel*  pucSrc, Int iSrcStride, Int iSizeY );
  typedef Void (*XMixSampleFunc)( XPel* pucDest, Int iDestStride, XPel* pucSrc, Int iSrcStride, Int iSizeY );

protected:
  SampleWeighting();
  virtual ~SampleWeighting()  {}

public:
  static ErrVal create( SampleWeighting*& rpcSampleWeighting );
  ErrVal destroy();
  virtual ErrVal init();
  ErrVal uninit();

  ErrVal initSlice( const SliceHeader& rcSliceHeader );

  Void getTargetBuffers( YuvMbBuffer* apcTarBuffer[2], YuvMbBuffer* pcRecBuffer, const PredWeight* pcPW0, const PredWeight* pcPW1 );

  Void weightLumaSamples(   YuvMbBuffer* pcRecBuffer, Int iSizeX, Int iSizeY, LumaIdx cIdx, const PredWeight* pcPW0, const PredWeight* pcPW1 );
  Void weightChromaSamples( YuvMbBuffer* pcRecBuffer, Int iSizeX, Int iSizeY, LumaIdx cIdx, const PredWeight* pcPW0, const PredWeight* pcPW1 );

  //===== for motion estimation of bi-predicted blocks with standard weights =====
  Void inverseLumaSamples  ( YuvMbBuffer* pcDesBuffer, YuvMbBuffer* pcOrgBuffer, YuvMbBuffer* pcFixBuffer, Int iYSize, Int iXSize );

  //===== for motion estimation of bi-predicted blocks with non-standard weights =====
  Void weightInverseLumaSamples  ( YuvMbBuffer* pcDesBuffer, YuvMbBuffer* pcOrgBuffer, YuvMbBuffer* pcFixBuffer, const PredWeight* pcSearchPW, const PredWeight* pcFixPW, Double&  rdWeight, Int iYSize, Int iXSize );

  //===== for motion estimation of unidirectional predicted blocks with non-standard weights =====
  Void weightInverseLumaSamples  ( YuvMbBuffer* pcDesBuffer, YuvMbBuffer* pcOrgBuffer, const PredWeight* pcPW, Double&  rdWeight, Int iYSize, Int iXSize );
  Void weightInverseChromaSamples( YuvMbBuffer* pcDesBuffer, YuvMbBuffer* pcOrgBuffer, const PredWeight* pcPW, Double* padWeight, Int iYSize, Int iXSize );

//TMM_WP
  ErrVal initSliceForWeighting( const SliceHeader& rcSliceHeader);

//TMM_WP

protected:
  Void xMixB      ( Pel*  pucDest, Int iDestStride, Pel*  pucSrc, Int iSrcStride, Int iSizeY, Int iSizeX );
  Void xMixB      ( XPel* pucDest, Int iDestStride, XPel* pucSrc, Int iSrcStride, Int iSizeY, Int iSizeX );
  Void xMixBWeight( Pel*  pucDest, Int iDestStride, Pel*  pucSrc, Int iSrcStride, Int iSizeY, Int iSizeX, Int iWD, Int iWS, Int iOffset, UInt uiDenom );
  Void xMixBWeight( XPel* pucDest, Int iDestStride, XPel* pucSrc, Int iSrcStride, Int iSizeY, Int iSizeX, Int iWD, Int iWS, Int iOffset, UInt uiDenom );
  Void xWeight    ( Pel*  pucDest, Int iDestStride,                               Int iSizeY, Int iSizeX, Int iWeight,      Int iOffset, UInt uiDenom );
  Void xWeight    ( XPel* pucDest, Int iDestStride,                               Int iSizeY, Int iSizeX, Int iWeight,      Int iOffset, UInt uiDenom );

private:
  static Void xMixB16x( Pel* pucDest, Int iDestStride, Pel* pucSrc, Int iSrcStride, Int iSizeY );
  static Void xMixB8x ( Pel* pucDest, Int iDestStride, Pel* pucSrc, Int iSrcStride, Int iSizeY );
  static Void xMixB4x ( Pel* pucDest, Int iDestStride, Pel* pucSrc, Int iSrcStride, Int iSizeY );
  static Void xXMixB16x( XPel* pucDest, Int iDestStride, XPel* pucSrc, Int iSrcStride, Int iSizeY );
  static Void xXMixB8x ( XPel* pucDest, Int iDestStride, XPel* pucSrc, Int iSrcStride, Int iSizeY );
  static Void xXMixB4x ( XPel* pucDest, Int iDestStride, XPel* pucSrc, Int iSrcStride, Int iSizeY );

protected:
  MixSampleFunc m_afpMixSampleFunc[5];
  XMixSampleFunc m_afpXMixSampleFunc[5];

private:
  YuvMbBuffer  m_cIntYuvBiBuffer;
  UInt            m_uiLumaLogWeightDenom;
  UInt            m_uiChromaLogWeightDenom;
  Bool            m_bExplicit;
  Bool            m_bWeightedPredDisableP;
  Bool            m_bWeightedPredDisableB;
};


H264AVC_NAMESPACE_END


#endif // !defined(AFX_SAMPLEWEIGHTING_H__6B0A73D2_EAF3_497F_B114_913D68E1C1C0__INCLUDED_)
