/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#if !defined(AFX_TRACEFILE_H__B87E26CF_023E_4DC7_8F94_D3E38F59ABA1__INCLUDED_)
#define AFX_TRACEFILE_H__B87E26CF_023E_4DC7_8F94_D3E38F59ABA1__INCLUDED_


#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000



#define ENCODER_TRACE     0
#define DECODER_TRACE     0

#define MAX_TRACE_LAYERS 17

#define MAX_LINE_LENGTH 255
#define MAX_BITS_LENGTH 64


H264AVC_NAMESPACE_BEGIN


class H264AVCCOMMONLIB_API TraceFile
{
public:
	TraceFile         ();
	virtual ~TraceFile();

  static ErrVal disable     ();
  static ErrVal enable      ();

  static ErrVal initTrace   ();
  static ErrVal openTrace   ( const Char* pucBaseFilename );
  static ErrVal closeTrace  ();
  static ErrVal setLayer    ( UInt  uiLayerId );


  static ErrVal startNalUnit();
  static ErrVal startFrame  ();
  static ErrVal startSlice  ();
  static ErrVal startMb     ( Int   iMbAddress  );

  static ErrVal printHeading( const Char* pcString    );

  static ErrVal countBits   ( UInt  uiBitCount );
  static ErrVal printPos    ();

  static ErrVal printString ( const Char* pcString );
  static ErrVal printVal    ( UInt  uiVal );
  static ErrVal printVal    ( Int   iVal );
  static ErrVal printXVal   ( UInt  uiVal );

  static ErrVal addBits     ( UInt  uiVal, UInt uiLength );
  static ErrVal printBits   ( UInt  uiVal, UInt uiLength );
  static ErrVal printCode   ( UInt  uiVal );
  static ErrVal printCode   ( Int   iVal );
  static ErrVal printType   ( const Char* pcString);

  static ErrVal newLine();

protected:
  static Bool  sm_bDisable;
  static UInt  sm_uiLayer;
  static FILE* sm_fTrace      [MAX_TRACE_LAYERS];
  static UInt  sm_uiFrameNum  [MAX_TRACE_LAYERS];
  static UInt  sm_uiSliceNum  [MAX_TRACE_LAYERS];
  static UInt  sm_uiPosCounter[MAX_TRACE_LAYERS];
  static Char  sm_acLine      [MAX_LINE_LENGTH];
  static Char  sm_acType      [9];
  static Char  sm_acPos       [9];
  static Char  sm_acCode      [6];
  static Char  sm_acBits      [MAX_BITS_LENGTH];
};



H264AVC_NAMESPACE_END




#if ENCODER_TRACE
  #define ETRACE_OFF       if( m_bTraceEnable ) TraceFile::disable     ()
  #define ETRACE_ON        if( m_bTraceEnable ) TraceFile::enable      ()

  #define INIT_ETRACE      if( m_bTraceEnable ) TraceFile::initTrace   ()
  #define OPEN_ETRACE      if( m_bTraceEnable ) TraceFile::openTrace   ("TraceEncoder")
  #define CLOSE_ETRACE     if( m_bTraceEnable ) TraceFile::closeTrace  ()

  #define ETRACE_LAYER(x)  if( m_bTraceEnable ) TraceFile::setLayer    (x)
  #define ETRACE_NEWFRAME  if( m_bTraceEnable ) TraceFile::startFrame  ()
  #define ETRACE_NEWSLICE  if( m_bTraceEnable ) TraceFile::startSlice  ()
  #define ETRACE_NEWMB(x)  if( m_bTraceEnable ) TraceFile::startMb     (x)
  #define ETRACE_HEADER(x)                      TraceFile::printHeading(x)

  #define ETRACE_POS       if( m_bTraceEnable ) TraceFile::printPos    ()
  #define ETRACE_COUNT(i)  if( m_bTraceEnable ) TraceFile::countBits   (i)

  #define ETRACE_BITS(v,l) if( m_bTraceEnable ) TraceFile::addBits     (v,l)
  #define ETRACE_CODE(v)   if( m_bTraceEnable ) TraceFile::printCode   (v)

  #define ETRACE_TH(t)     if( m_bTraceEnable ) TraceFile::printString (t)
  #define ETRACE_T(t)      if( m_bTraceEnable ) TraceFile::printString (t)
  #define ETRACE_TY(t)     if( m_bTraceEnable ) TraceFile::printType   (t)
  #define ETRACE_V(t)      if( m_bTraceEnable ) TraceFile::printVal    (t)
  #define ETRACE_X(t)      if( m_bTraceEnable ) TraceFile::printXVal   (t)

  #define ETRACE_N         if( m_bTraceEnable ) TraceFile::newLine     ()
  #define ETRACE_DO(x)     if( m_bTraceEnable ) x
  #define ETRACE_DECLARE(x) x
 #else

  #define ETRACE_OFF
  #define ETRACE_ON

  #define OPEN_ETRACE
  #define INIT_ETRACE
  #define CLOSE_ETRACE

  #define ETRACE_LAYER(x)
  #define ETRACE_NEWFRAME
  #define ETRACE_NEWSLICE
  #define ETRACE_NEWMB(x)
  #define ETRACE_HEADER(x)

  #define ETRACE_POS
  #define ETRACE_COUNT(i)

  #define ETRACE_BITS(v,l)
  #define ETRACE_CODE(v)

  #define ETRACE_TH(t)
  #define ETRACE_T(t)
  #define ETRACE_TY(t)
  #define ETRACE_V(t)
  #define ETRACE_X(t)

  #define ETRACE_N
  #define ETRACE_DO(x)
  #define ETRACE_DECLARE(x)
#endif

#if DECODER_TRACE
  #define DTRACE_OFF       TraceFile::disable     ()
  #define DTRACE_ON        TraceFile::enable      ()

  #define INIT_DTRACE      TraceFile::initTrace   ()
  #define OPEN_DTRACE      TraceFile::openTrace   ("TraceDecoder")
  #define CLOSE_DTRACE     TraceFile::closeTrace  ()

  #define DTRACE_LAYER(x)  TraceFile::setLayer    (x)
  #define DTRACE_NEWFRAME  TraceFile::startFrame  ()
  #define DTRACE_NEWSLICE  TraceFile::startSlice  ()
  #define DTRACE_NEWMB(x)  TraceFile::startMb     (x)
  #define DTRACE_HEADER(x) TraceFile::printHeading(x)

  #define DTRACE_POS       TraceFile::printPos    ()
  #define DTRACE_COUNT(i)  TraceFile::countBits   (i)

  #define DTRACE_BITS(v,l) TraceFile::addBits     (v,l)
  #define DTRACE_CODE(v)   TraceFile::printCode   (v)

  #define DTRACE_TH(t)     TraceFile::printString (t)
  #define DTRACE_T(t)      TraceFile::printString (t)
  #define DTRACE_TY(t)     TraceFile::printType   (t)
  #define DTRACE_V(t)      TraceFile::printVal    (t)
  #define DTRACE_X(t)      TraceFile::printXVal   (t)

  #define DTRACE_N         TraceFile::newLine     ()
  #define DTRACE_DO(x)     x
#else
  #define DTRACE_OFF
  #define DTRACE_ON

  #define OPEN_DTRACE
  #define OPEN_DTRACE_NR
  #define INIT_DTRACE
  #define CLOSE_DTRACE

  #define DTRACE_LAYER(x)
  #define DTRACE_NEWFRAME
  #define DTRACE_NEWSLICE
  #define DTRACE_NEWMB(x)
  #define DTRACE_HEADER(x)

  #define DTRACE_POS
  #define DTRACE_COUNT(i)

  #define DTRACE_BITS(v,l)
  #define DTRACE_CODE(v)

  #define DTRACE_TH(t)
  #define DTRACE_T(t)
  #define DTRACE_TY(t)
  #define DTRACE_V(t)
  #define DTRACE_X(t)

  #define DTRACE_N
  #define DTRACE_DO(x)
#endif

#endif // !defined(AFX_TRACEFILE_H__B87E26CF_023E_4DC7_8F94_D3E38F59ABA1__INCLUDED_)
