/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#if !defined(AFX_INVERSETRANSFORM_H__B2D732EC_10EA_4C2F_9387_4456CFCA4439__INCLUDED_)
#define AFX_INVERSETRANSFORM_H__B2D732EC_10EA_4C2F_9387_4456CFCA4439__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "H264AVCCommonLib/Quantizer.h"
#include "H264AVCCommonLib/YuvMbBuffer.h"

H264AVC_NAMESPACE_BEGIN

class H264AVCCOMMONLIB_API CoeffLevelPred
{
protected:
	CoeffLevelPred();
	virtual ~CoeffLevelPred();

public:

  ErrVal ScaleCoeffLevels       ( TCoeff* piCoeff,
                                  TCoeff* piRefCoeff,
                                  UInt uiQp,
                                  UInt uiRefQp,
                                  UInt uiNumCoeff );

};

class H264AVCCOMMONLIB_API Transform :
public Quantizer
, CoeffLevelPred
{
protected:
	Transform();
	virtual ~Transform();

public:
  static ErrVal create( Transform*& rpcTransform );
  ErrVal destroy();
  Void setClipMode( Bool bEnableClip ) { m_bClip = bEnableClip; }

  Bool getClipMode()                   { return m_bClip; }

  // JVT-V035 functions for SVC to AVC rewrite
  ErrVal        predict4x4Blk             ( TCoeff* piCoeff, TCoeff* piRef, UInt uiRefQp, UInt& ruiAbsSum );
  ErrVal        predict8x8Blk             ( TCoeff* piCoeff, TCoeff* piRef, UInt uiRefQp, UInt& ruiAbsSum );
  ErrVal        predictChromaBlocks       ( TCoeff* piCoeff, TCoeff* piRef, UInt uiRefQp, UInt& ruiDcAbs, UInt& ruiAcAbs );
  ErrVal        predictScaledACCoeffs     ( TCoeff *piCoeff, TCoeff* piRef, UInt uiRefQp, const UChar* pucScale );
  ErrVal        predictScaledChromaCoeffs ( TCoeff *piCoeff, TCoeff* piRef, UInt uiRefQp, const UChar* pucScaleCb, const UChar* pucScaleCr );
  ErrVal        predictMb16x16            ( TCoeff* piCoeff, TCoeff* piRef, UInt uiRefQp, UInt& ruiDcAbs, UInt& ruiAcAbs );
  ErrVal        addPrediction4x4Blk       ( TCoeff* piCoeff, TCoeff* piRefCoeff, UInt uiDstQp, UInt uiRefQp, UInt &uiCoded  );
  ErrVal        addPrediction8x8Blk       ( TCoeff* piCoeff, TCoeff* piRefCoeff, UInt uiQp, UInt uiRefQp, Bool& bCoded  );
  ErrVal        addPredictionChromaBlocks ( TCoeff* piCoeff, TCoeff* piRef, UInt uiQp, UInt uiRefQp, Bool& bDCflag, Bool& bACflag );

  ErrVal        transform8x8Blk           ( YuvMbBuffer*       pcOrgData,
                                            YuvMbBuffer*       pcPelData,
                                            TCoeff*               piCoeff,
                                            const UChar*          pucScale,
                                            UInt&                 ruiAbsSum );
  ErrVal        transform4x4Blk           ( YuvMbBuffer*       pcOrgData,
                                            YuvMbBuffer*       pcPelData,
                                            TCoeff*               piCoeff,
                                            const UChar*          pucScale,
                                            UInt&                 ruiAbsSum );
  ErrVal        transformMb16x16          ( YuvMbBuffer* pcOrgData,
                                            YuvMbBuffer* pcPelData,
                                            TCoeff* piCoeff,
                                            const UChar* pucScale,
                                            UInt& ruiDcAbs,
                                            UInt& ruiAcAbs );
  ErrVal        transformChromaBlocks     ( XPel* pucOrg,
                                            XPel* pucRec,
                                            const CIdx            cCIdx,
                                            Int iStride,
                                            TCoeff* piCoeff,
                                            TCoeff* piQuantCoeff,
                                            const UChar* pucScale,
                                            UInt& ruiDcAbs,
                                            UInt& ruiAcAbs );

  ErrVal        invTransform8x8Blk        ( XPel* puc, Int iStride, TCoeff* piCoeff );
  ErrVal        invTransform4x4Blk        ( XPel* puc, Int iStride, TCoeff* piCoeff );
  ErrVal        invTransformChromaBlocks  ( XPel* puc, Int iStride, TCoeff* piCoeff );

  ErrVal        invTransform4x4Blk        ( Pel*  puc, Int iStride, TCoeff* piCoeff );
  ErrVal        invTransformChromaBlocks  ( Pel*  puc, Int iStride, TCoeff* piCoeff );

  ErrVal        invTransformDcCoeff       ( TCoeff* piCoeff, const Int iQpScale, const Int iQpPer );
  ErrVal        invTransformDcCoeff       ( TCoeff* piCoeff, Int iQpScale );
  Void          invTransformChromaDc      ( TCoeff* piCoeff );

  void         setStoreCoeffFlag(Bool flag)   { m_storeCoeffFlag = flag;}
  Bool         getStoreCoeffFlag()        {return  m_storeCoeffFlag;}
  ErrVal       transform4x4BlkCGS         ( YuvMbBuffer*         pcOrgData,
                                            YuvMbBuffer*         pcPelData,
                                            TCoeff*                 piCoeff,
                                            TCoeff*                 piCoeffBase,
                                            const UChar*            pucScale,
                                            UInt&                   ruiAbsSum );
  ErrVal       transform8x8BlkCGS           ( YuvMbBuffer*       pcOrgData,
                                            YuvMbBuffer*         pcPelData,
                                            TCoeff*                 piCoeff,
                                            TCoeff*                 piCoeffBase,
                                            const UChar*            pucScale,
                                            UInt&                   ruiAbsSum );

  ErrVal        transformChromaBlocksCGS  ( XPel* pucOrg,
                                            XPel* pucRec,
                                            const CIdx            cCIdx,
                                            Int iStride,
                                            TCoeff* piCoeff,
                                            TCoeff* piQuantCoeff,
                                            TCoeff* piCoeffBase,
                                            const UChar* pucScale,
                                            UInt& ruiDcAbs,
                                            UInt& ruiAcAbs );

private:
  Void xForTransform8x8Blk      ( XPel* pucOrg, XPel* pucRec, Int iStride, TCoeff* piPredCoeff );
  Void xForTransform4x4Blk      ( XPel* pucOrg, XPel* pucRec, Int iStride, TCoeff* piPredCoeff );

  Void xInvTransform4x4Blk      ( XPel* puc, Int iStride, TCoeff* piCoeff );
  Void xInvTransform4x4Blk      ( Pel*  puc, Int iStride, TCoeff* piCoeff );

  Void xInvTransform4x4BlkNoAc  ( XPel* puc, Int iStride, TCoeff* piCoeff );
  Void xInvTransform4x4BlkNoAc  ( Pel*  puc, Int iStride, TCoeff* piCoeff );

  Void xForTransformChromaDc    ( TCoeff* piCoeff );
  Void xForTransformLumaDc      ( TCoeff* piCoeff );

  Int  xRound                   ( Int i     )             { return ((i)+(1<<5))>>6; }
  Int  xClip                    ( Int iPel  )             { return ( m_bClip ? gClip( iPel ) : iPel); }

  Void xQuantDequantUniform8x8      ( TCoeff* piQCoeff,
                                      TCoeff* piCoeff,
                                      const QpParameter& rcQp,
                                      const UChar* pucScale,
                                      UInt& ruiAbsSum );
  Void xQuantDequantUniform4x4      ( TCoeff* piQCoeff,
                                      TCoeff* piCoeff,
                                      const QpParameter& rcQp,
                                      const UChar* pucScale,
                                      UInt& ruiAbsSum );
  Void xQuantDequantNonUniformLuma  ( TCoeff* piQCoeff,
                                      TCoeff* piCoeff,
                                      const QpParameter& rcQp,
                                      const UChar* pucScale,
                                      UInt& ruiDcAbs,
                                      UInt& ruiAcAbs );
  Void xQuantDequantNonUniformChroma( TCoeff* piQCoeff,
                                      TCoeff* piCoeff,
                                      const QpParameter& rcQp,
                                      const UChar* pucScale,
                                      UInt& ruiDcAbs,
                                      UInt& ruiAcAbs );

  Void x4x4DequantChroma  ( TCoeff*             piQCoeff,
                            TCoeff*             piCoeff,
                            const QpParameter&  rcQp,
                            const UChar*        pucScale );

protected:
  const SliceHeader*  m_pcSliceHeader;
  Bool                m_bClip;
  Bool                m_storeCoeffFlag;
};


H264AVC_NAMESPACE_END

#endif // !defined(AFX_INVERSETRANSFORM_H__B2D732EC_10EA_4C2F_9387_4456CFCA4439__INCLUDED_)
