/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <memory.h>
#include <limits.h>

#ifndef _RATE_CTL_H_
#define _RATE_CTL_H_

#define RC_MODE_1              1
#define RC_MODE_2              2
#define MB_BLOCK_SIZE         16

extern bool bRateControlEnable;

#if defined(WIN32) || (__STDC_VERSION__ >= 199901L)

static inline int imin(int a, int b)
{
  return ((a) < (b)) ? (a) : (b);
}

static inline int imax(int a, int b)
{
  return ((a) > (b)) ? (a) : (b);
}

static inline int iClip3(int low, int high, int x)
{
  x = imax(x, low);
  x = imin(x, high);

  return x;
}

static inline int iabs(int x)
{
  return ((x) < 0) ? -(x) : (x);
}

#else

#define imin(a, b)                  (((a) < (b)) ? (a) : (b))
#define imax(a, b)                  (((a) > (b)) ? (a) : (b))
#define iClip3(low, high, x)        (imax( imin(x, high), low))
#define iabs(x)                     (((x) < 0)   ? -(x) : (x))

#endif

typedef enum
{
  FRAME_CODING,
  FIELD_CODING,
  ADAPTIVE_CODING
} CodingType;


class jsvm_parameters {
public:
  float FrameRate;
	float bit_rate;

	int current_mb_nr;
	int current_frame_number;
	int successive_Bframe;
	int jumpd;
	int Frame_Total_Number_MB;
	int basicunit;
	int height;
	int width;
	int FrameSizeInMbs;
	int PicInterlace;
	int MbInterlace;
	int channel_type;
	int type;
	int BasicUnit;
	int frame_mbs_only_flag;
	int intra_period;
	int SetInitialQP;
	int NumberofCodedMacroBlocks;
  int qp;
  int RCMaxQP;
  int RCMinQP;
  int RCUpdateMode;
  int number;
  int PicWidthInMbs;
  int size;
  int HierarchicalCoding;
  int no_frames;
  int CurrGopLevel;
  int nal_reference_idc;
  int HierarchicalLevels;

  unsigned int m_uiLayerId;
  unsigned int m_uiIntraPeriod;
};

class rc_generic {
public:
  // RC flags
  int   m_iTopFieldFlag;
  int   m_iFieldControl;
  int   m_iFieldFrame;
  int   m_iNoGranularFieldRC;
  // bits stats
  int   m_iNumberofHeaderBits;
  int   m_iNumberofTextureBits;
  int   m_iNumberofBasicUnitHeaderBits;
  int   m_iNumberofBasicUnitTextureBits;
  // frame stats
  int   m_iNumberofGOP;
  int   m_iNumberofCodedBFrame;
  // MAD stats
  Int64 m_i64TotalMADBasicUnit;
  int   *m_piMADofMB;
  // buffer and budget
  Int64 m_i64CurrentBufferFullness; //LIZG 25/10/2002
  int   m_iRemainingBits;

  int   m_iRCTextureBits;
  int   m_iRCHeaderBits;
  int   m_iRCTotalBits;

  jsvm_parameters *m_pcJSVMParams;

  rc_generic( jsvm_parameters *jsvm_params );
  ~rc_generic( void );

  int    Qstep2QP( double dQstep );
  double QP2Qstep( int iQP );
  double ComputeFrameMAD( void );
  void   generic_alloc( void );
  void   generic_free( void );

  void   update_rc( unsigned int currentMAD );
  int    getCurrGopLevel( int frame_no );
  void   adaptInitialQP( void );
};

#endif
