/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#if !defined(AFX_BITREADBUFFER_H__F1308E37_7998_4953_9F78_FF6A3DBC22B5__INCLUDED_)
#define AFX_BITREADBUFFER_H__F1308E37_7998_4953_9F78_FF6A3DBC22B5__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

H264AVC_NAMESPACE_BEGIN

class BitReadBuffer
{
protected:
  BitReadBuffer();
  virtual ~BitReadBuffer();

public:
  static ErrVal create( BitReadBuffer*& rpcBitReadBuffer );
  ErrVal destroy();
  ErrVal init   () { return Err::m_nOK; }
  ErrVal uninit () { return Err::m_nOK; }

  ErrVal  initPacket  ( ULong*  puiBits,
                        UInt    uiBitsInPacket );
  Void    setModeCabac()
  { // problem with cabac, cause cabac decoder uses stop bit + trailing stuffing bits
    m_uiBitsLeft = 8 * ( ( m_uiBitsLeft + 7 ) / 8 );
  }

  ErrVal  get     ( UInt& ruiBits, UInt uiNumberOfBits );
  ErrVal  get     ( UInt& ruiBits );
  ErrVal  flush   ( UInt uiNumberOfBits );
  ErrVal  pcmSamples ( TCoeff* pCoeff, UInt uiNumberOfSamples );

  Int     getBitsUntilByteAligned ()  { return m_iValidBits & 0x7;  }
  Bool    isWordAligned           ()  { return ( 0 == ( m_iValidBits & 0x1f ) ); }
  Bool    isByteAligned           ()  { return ( 0 == ( m_iValidBits & 0x7  ) ); }
  Bool    isValid                 ()  { return ( m_uiBitsLeft > 1 ); }

private:
  __inline Void xReadNextWord();

  ULong  xSwap( ULong ul )
  {
#ifdef MSYS_BIG_ENDIAN
    return ul;
#else
    ULong ul2;
    ul2  = ul>>24;
    ul2 |= (ul>>8) & 0x0000ff00;
    ul2 |= (ul<<8) & 0x00ff0000;
    ul2 |= ul<<24;
    return ul2;
#endif
  }

protected:
  UInt   m_uiDWordsLeft;
  UInt   m_uiBitsLeft;
  Int    m_iValidBits;
  ULong  m_ulCurrentBits;
  UInt   m_uiNextBits;
  ULong* m_pulStreamPacket;
};


H264AVC_NAMESPACE_END

#endif // !defined(AFX_BITREADBUFFER_H__F1308E37_7998_4953_9F78_FF6A3DBC22B5__INCLUDED_)
