/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#if !defined(AFX_H264AVCDECODER_H__FBF0345F_A5E5_4D18_8BEC_4A68790901F7__INCLUDED_)
#define AFX_H264AVCDECODER_H__FBF0345F_A5E5_4D18_8BEC_4A68790901F7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "GOPDecoder.h"
#include "H264AVCCommonLib/Sei.h"
#include "H264AVCCommonLib/MotionCompensation.h"
#include "H264AVCCommonLib/LoopFilter.h"

H264AVC_NAMESPACE_BEGIN

class SliceReader;
class SliceDecoder;
class PocCalculator;
class LoopFilter;
class HeaderSymbolReadIf;
class ParameterSetMng;
class NalUnitParser;
class ControlMngIf;
class AccessUnit;
class NALUnit;
class NonVCLNALUnit;


class H264AVCDECODERLIB_API H264AVCDecoder
{
protected:
	H264AVCDecoder         ();
  virtual ~H264AVCDecoder();

public:
  //===== creation and initialization =====
  static  ErrVal  create  ( H264AVCDecoder*&    rpcH264AVCDecoder );
  ErrVal  destroy         ();
  ErrVal  init            ( NalUnitParser*      pcNalUnitParser,
                            HeaderSymbolReadIf* pcHeaderSymbolReadIf,
                            ParameterSetMng*    pcParameterSetMngAUInit,
                            ParameterSetMng*    pcParameterSetMngDecode,
                            LayerDecoder*       apcLayerDecoder[MAX_LAYERS] );
  ErrVal  uninit          ();

  //===== main processing functions =====
  ErrVal  initNALUnit     ( BinData*&         rpcBinData,
                            AccessUnit&       rcAccessUnit );
  ErrVal  processNALUnit  ( PicBuffer*        pcPicBuffer,
                            PicBufferList&    rcPicBufferOutputList,
                            PicBufferList&    rcPicBufferUnusedList,
                            BinDataList&      rcBinDataList,
                            NALUnit&          rcNALUnit );

  //===== update decoded picture buffer of all layers =====
  ErrVal  updateDPB       ( UInt              uiTargetDependencyId,
                            PicBufferList&    rcPicBufferOutputList,
                            PicBufferList&    rcPicBufferUnusedList );

  //===== get inter-layer prediction data =====
  ErrVal  getBaseLayerData              ( SliceHeader&      rcELSH,
                                          Frame*&           pcFrame,
                                          Frame*&           pcResidual,
                                          MbDataCtrl*&      pcMbDataCtrl,
                                          ResizeParameters& rcResizeParameters,
                                          UInt              uiBaseLayerId );
  ErrVal  getBaseSliceHeader            ( SliceHeader*&     rpcSliceHeader,
                                          UInt              uiRefLayerDependencyId );

protected:
  ErrVal  xProcessNonVCLNALUnit         ( NonVCLNALUnit&    rcNonVCLNALUnit );


protected:
  Bool                m_bInitDone;
  NalUnitParser*      m_pcNalUnitParser;
  HeaderSymbolReadIf* m_pcHeaderSymbolReadIf;
  ParameterSetMng*    m_pcParameterSetMngAUInit;
  ParameterSetMng*    m_pcParameterSetMngDecode;
  LayerDecoder*       m_apcLayerDecoder[MAX_LAYERS];
  UInt                m_auiLastDQTPId[4]; // only for setting correct parameters in filler data instances
};

H264AVC_NAMESPACE_END

#endif // !defined(AFX_H264AVCDECODER_H__FBF0345F_A5E5_4D18_8BEC_4A68790901F7__INCLUDED_)
