/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#if !defined(AFX_BITCOUNTER_H__5062A2A9_6E04_4211_A337_9BDED6D88A82__INCLUDED_)
#define AFX_BITCOUNTER_H__5062A2A9_6E04_4211_A337_9BDED6D88A82__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BitWriteBufferIf.h"


H264AVC_NAMESPACE_BEGIN

class BitCounter :
public BitWriteBufferIf
{
public:

  BitWriteBufferIf* getNextBitWriteBuffer( Bool bStartNewBitstream ) { return NULL; }
  Bool              nextBitWriteBufferActive() { return false; }
  UChar*            getNextBuffersPacket()  { return NULL; }

  static ErrVal create( BitCounter*& rpcBitCounter );
  ErrVal destroy();

  ErrVal init()                                           { m_uiBitCounter = 0; return Err::m_nOK; }
  ErrVal uninit()                                         { m_uiBitCounter = 0; return Err::m_nOK; }

  BitCounter();
  virtual ~BitCounter();
  ErrVal write( UInt uiBits, UInt uiNumberOfBits = 1)     { m_uiBitCounter += uiNumberOfBits; return Err::m_nOK; }

  ErrVal pcmSamples( const TCoeff* pCoeff, UInt uiNumberOfSamples ) { m_uiBitCounter+=8*uiNumberOfSamples; return Err::m_nERR; }

  UInt getNumberOfWrittenBits()                           { return m_uiBitCounter; }

  Bool isByteAligned()                                    { return (0 == (m_uiBitCounter & 0x03)); }
  Bool isWordAligned()                                    { return (0 == (m_uiBitCounter & 0x1f)); }

  ErrVal writeAlignZero()                                 { return Err::m_nERR; }
  ErrVal writeAlignOne()                                  { return Err::m_nERR; }
  ErrVal flushBuffer()                                    { m_uiBitCounter = 0; return Err::m_nOK; }

  ErrVal   getLastByte(UChar &uiLastByte, UInt &uiLastBitPos) { return Err::m_nERR;} //FIX_FRAG_CAVLC

	//JVT-X046 {
	void loadBitWriteBuffer(BitWriteBufferIf* pcBitWriteBufferIf)	{}
	void loadBitCounter(BitWriteBufferIf* pcBitWriteBufferIf)
	{
		BitCounter* pcBitCounter = (BitCounter*)(pcBitWriteBufferIf);
		m_uiBitCounter = pcBitCounter->getNumberOfWrittenBits();
	}
	UInt getBitsWritten(void) { return 0; }
	//JVT-X046 }

private:
  UInt m_uiBitCounter;
};


H264AVC_NAMESPACE_END

#endif // !defined(AFX_BITCOUNTER_H__5062A2A9_6E04_4211_A337_9BDED6D88A82__INCLUDED_)
