/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#if !defined(AFX_CABAENCODER_H__19223BF1_DA9C_475F_9AEE_D1A55237EB1E__INCLUDED_)
#define AFX_CABAENCODER_H__19223BF1_DA9C_475F_9AEE_D1A55237EB1E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BitWriteBufferIf.h"


H264AVC_NAMESPACE_BEGIN


class CabacContextModel;

class CabaEncoder
{
protected:
  CabaEncoder();
  virtual ~CabaEncoder();

public:
  ErrVal start();
  ErrVal getLastByte(UChar &uiLastByte, UInt &uiLastBitPos); //FIX_FRAG_CAVLC
  ErrVal setFirstBits(UChar ucByte,UInt uiLastBitPos); //FIX_FRAG_CAVLC
  ErrVal init( BitWriteBufferIf* pcBitWriteBufferIf );
  ErrVal uninit();

  ErrVal writeEPSymbol( UInt uiSymbol );
  ErrVal writeSymbol( UInt uiSymbol, CabacContextModel& rcCCModel );
  ErrVal writeUnaryMaxSymbol( UInt uiSymbol, CabacContextModel* pcCCModel, Int iOffset, UInt uiMaxSymbol );
  ErrVal writeUnarySymbol( UInt uiSymbol, CabacContextModel* pcCCModel, Int iOffset );

  ErrVal writeExGolombLevel( UInt uiSymbol, CabacContextModel& rcCCModel  );
  ErrVal writeEpExGolomb( UInt uiSymbol, UInt uiCount );
  ErrVal writeExGolombMvd( UInt uiSymbol, CabacContextModel* pcCCModel, UInt uiMaxBin );
  ErrVal writeTerminatingBit( UInt uiBit );
  ErrVal finish();
  UInt   getWrittenBits()  { return m_pcBitWriteBufferIf->getNumberOfWrittenBits() + 8 + m_uiBitsToFollow - m_uiBitsLeft + 1; } //JVT-P031

  Void   setStates  (CabaEncoder* pcExtEncoder )
  {
    m_pcBitWriteBufferIf  = pcExtEncoder->m_pcBitWriteBufferIf;
    m_uiRange             = pcExtEncoder->m_uiRange;
    m_uiLow               = pcExtEncoder->m_uiLow;
    m_uiByte              = pcExtEncoder->m_uiByte;
    m_uiBitsLeft          = pcExtEncoder->m_uiBitsLeft;
    m_uiBitsToFollow      = pcExtEncoder->m_uiBitsToFollow;
  }
  Void   getStates  (CabaEncoder* pcExtEncoder )
  {
    pcExtEncoder->m_pcBitWriteBufferIf  = m_pcBitWriteBufferIf;
    pcExtEncoder->m_uiRange             = m_uiRange;
    pcExtEncoder->m_uiLow               = m_uiLow;
    pcExtEncoder->m_uiByte              = m_uiByte;
    pcExtEncoder->m_uiBitsLeft          = m_uiBitsLeft;
    pcExtEncoder->m_uiBitsToFollow      = m_uiBitsToFollow;
  }
	BitWriteBufferIf* getBitWriteBufferIf(void){return m_pcBitWriteBufferIf;}//JVT-X046

private:
  __inline ErrVal xWriteBit( UInt uiBit);
  __inline ErrVal xWriteBitAndBitsToFollow( UInt uiBit);

protected:
  BitWriteBufferIf* m_pcBitWriteBufferIf;

  UInt m_uiRange;
  UInt m_uiLow;
  UInt m_uiByte;
  UInt m_uiBitsLeft;
  UInt m_uiBitsToFollow;
  Bool m_bTraceEnable;
};


H264AVC_NAMESPACE_END


#endif // !defined(AFX_CABAENCODER_H__19223BF1_DA9C_475F_9AEE_D1A55237EB1E__INCLUDED_)
