/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#include "H264AVCEncoderLib.h"
#include "H264AVCCommonLib.h"

#include "InputPicBuffer.h"


H264AVC_NAMESPACE_BEGIN


InputAccessUnit::InputAccessUnit( UInt        uiContFrameNumber,
                                  PicBuffer*  pcInputPicBuffer )
: m_uiContFrameNumber ( uiContFrameNumber )
, m_pcInputPicBuffer  ( pcInputPicBuffer  )
{
}

InputAccessUnit::~InputAccessUnit()
{
}




InputPicBuffer::InputPicBuffer()
: m_bInit ( false )
{
}

InputPicBuffer::~InputPicBuffer()
{
  uninit();
}

ErrVal
InputPicBuffer::create( InputPicBuffer*& rpcInputPicBuffer )
{
  rpcInputPicBuffer = new InputPicBuffer;
  ROF( rpcInputPicBuffer );
  return Err::m_nOK;
}

ErrVal
InputPicBuffer::destroy()
{
  delete this;
  return Err::m_nOK;
}

ErrVal
InputPicBuffer::init()
{
  ROF( m_cInputUnitList.empty() );
  m_uiContFrameNumber = 0;
  m_bInit             = true;
  return Err::m_nOK;
}

ErrVal
InputPicBuffer::uninit()
{
  ROF( m_cInputUnitList.empty() );
  m_bInit = false;
  return Err::m_nOK;
}

Bool
InputPicBuffer::empty()
{
  return m_cInputUnitList.empty();
}

ErrVal
InputPicBuffer::add( PicBuffer* pcInputPicBuffer )
{
  ROF( m_bInit );

  //===== add to list =====
  InputAccessUnit* pcInputAccessUnit = new InputAccessUnit( m_uiContFrameNumber++, pcInputPicBuffer );
  m_cInputUnitList.push_back( pcInputAccessUnit );

  return Err::m_nOK;
}


InputAccessUnit*
InputPicBuffer::remove( UInt  uiContFrameNumber )
{
  InputAccessUnit*                      pcIAU = NULL;
  std::list<InputAccessUnit*>::iterator iIter = m_cInputUnitList.begin();
  std::list<InputAccessUnit*>::iterator iEnd  = m_cInputUnitList.end  ();
  for( ; iIter != iEnd; iIter++ )
  {
    if( (*iIter)->getContFrameNumber() == uiContFrameNumber )
    {
      pcIAU = *iIter;
      m_cInputUnitList.erase( iIter );
      break;
    }
  }
  return pcIAU;
}

H264AVC_NAMESPACE_END



