/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#if !defined(AFX_MOTIONESTIMATIONCOST_H__46CFF00D_F656_4EA9_B8F8_81CC5610D443__INCLUDED_)
#define AFX_MOTIONESTIMATIONCOST_H__46CFF00D_F656_4EA9_B8F8_81CC5610D443__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "RateDistortionIf.h"


H264AVC_NAMESPACE_BEGIN


class MotionEstimationCost
{
protected:
  MotionEstimationCost();
  virtual ~MotionEstimationCost();

  ErrVal xUninit();

  Void xGetMotionCost( Bool bSad, Int iAdd )   { m_uiCost = m_pcRateDistortionIf->getMotionCostShift( bSad ) + iAdd;  }
  ErrVal xInitRateDistortionModel( Int iSubPelSearchLimit, RateDistortionIf* pcRateDistortionIf );

  UInt xGetVerCost( UInt y ) {  return (m_uiCost * m_puiVerCost[ y << m_iCostScale ] ) >> 16;  }
  UInt xGetHorCost( UInt x ) {  return (m_uiCost * m_puiHorCost[ x << m_iCostScale] ) >> 16;  }

  UInt xGetCost( UInt b )  { return ( m_uiCost * b ) >> 16; }

  UInt xGetCost( Mv& rcMv )
  {
    return ( m_uiCost * xGetBits( rcMv.getHor(), rcMv.getVer() ) ) >> 16;
  }
  UInt xGetCost( Int x, Int y )  {  return ( m_uiCost * xGetBits( x, y ) ) >> 16;  }
  UInt xGetBits( Int x, Int y )  {  return m_puiHorCost[ x << m_iCostScale] + m_puiVerCost[ y << m_iCostScale ];  }

  Void xSetPredictor( const Mv& rcMv )
  {
    m_puiHorCost = m_puiComponentCost - rcMv.getHor();
    m_puiVerCost = m_puiComponentCost - rcMv.getVer();
  }

  Void xSetCostScale( Int iCostScale ) { m_iCostScale = iCostScale; }
  UInt getComponentBits( Int iVal );

protected:
  RateDistortionIf* m_pcRateDistortionIf;
  UInt* m_puiComponentCostOriginP;
  UInt* m_puiComponentCost;
  UInt* m_puiVerCost;
  UInt* m_puiHorCost;
  UInt  m_uiCost;
  Int   m_iCostScale;
  Int m_iSearchLimit;
};


H264AVC_NAMESPACE_END


#endif // !defined(AFX_MOTIONESTIMATIONCOST_H__46CFF00D_F656_4EA9_B8F8_81CC5610D443__INCLUDED_)
