/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#include "H264AVCEncoderLib.h"
#include "CodingParameter.h"
#include "math.h"
#include "RateDistortion.h"

H264AVC_NAMESPACE_BEGIN


RateDistortion::RateDistortion():
  m_uiCostFactorMotionSAD( 0 ),
  m_uiCostFactorMotionSSE( 0 )
{
}

RateDistortion::~RateDistortion()
{
}

ErrVal RateDistortion::create( RateDistortion *&rpcRateDistortion )
{
  rpcRateDistortion = new RateDistortion;

  ROT( NULL == rpcRateDistortion );

  return Err::m_nOK;
}

Double RateDistortion::getCost( UInt uiBits, UInt uiDistortion )
{
  Double d = ((Double)uiDistortion + (Double)(Int)uiBits * m_dCost+.5);
  return (Double)(UInt)floor(d);
}

Double RateDistortion::getFCost( UInt uiBits, UInt uiDistortion )
{
  Double d = (((Double)uiDistortion) + ((Double)uiBits * m_dCost));
  return d;
}


ErrVal RateDistortion::destroy()
{
  delete this;
  return Err::m_nOK;
}


ErrVal RateDistortion::fixMacroblockQP( MbDataAccess& rcMbDataAccess )
{
  if( !( rcMbDataAccess.getMbData().getMbCbp() || rcMbDataAccess.getMbData().isIntra16x16() ) ) // has no coded texture
  {
    rcMbDataAccess.getMbData().setQp( rcMbDataAccess.getLastQp() );
  }
  return Err::m_nOK;
}


ErrVal
RateDistortion::setMbQpLambda( MbDataAccess& rcMbDataAccess, UInt uiQp, Double dLambda )
{
  rcMbDataAccess.getMbData().setQp( uiQp );

  m_dCost                 = dLambda;
  m_dSqrtCost             = sqrt( dLambda );
  m_uiCostFactorMotionSAD = (UInt)floor(65536.0 * sqrt( m_dCost ));
  m_uiCostFactorMotionSSE = (UInt)floor(65536.0 *       m_dCost  );

  return Err::m_nOK;
}


H264AVC_NAMESPACE_END
