/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#if !defined(AFX_SCHEDULER_H__4242CFD4_A40A_4FCE_B740_60624D030E86__INCLUDED_)
#define AFX_SCHEDULER_H__4242CFD4_A40A_4FCE_B740_60624D030E86__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "H264AVCCommonLib/Sei.h"
#include "CodingParameter.h"

// h264 namespace begin
H264AVC_NAMESPACE_BEGIN

// #define DEBUG_SCHEDULER
#define DEBUG_SCHEDULER_FILE "scheduler.txt"

class Scheduler
{
  class TimingUnit
  {
  public:

    Int64   m_iDataLength; // in Bytes
    Int64   m_iBitRate;    // in Bits
    Double  m_dInitialArrivalEarliest;
    Double  m_dInitialArrival;
    Double  m_dFinalArrival;
    Double  m_dRemoval;
    UInt    m_uiFirstIrd;
    UInt    m_uiIrd;
    UInt    m_uiIrdOffset;
    Bool    m_bCbr;

    TimingUnit();
    ErrVal  calcTiming( UInt uiSize, Double dTime, Bool bIsIdr);
    ErrVal  calcIrd(Double dTime);
  };


protected:
        Scheduler();
  virtual ~Scheduler() {};

public:
  static ErrVal create( Scheduler*& rpcScheduler );
  ErrVal destroy();

  ErrVal createBufferingSei( SEI::BufferingPeriod*& rpcBufferingPeriod, ParameterSetMng* pcParameterSetMng, UInt uiDQId );
  ErrVal createTimingSei( SEI::PicTiming*& rpcPicTiming, const VUI* pcVui, UInt uiPicNumOffset, SliceHeader &rcSH, UInt uiInputFormat, UInt uiLayerIndex);

  ErrVal calculateTiming( UInt uiVclSize, UInt uiAUSize, Bool bIsIdr, Bool bFieldPicFlag);
//  ErrVal calculateIrd();
  ErrVal init( CodingParameter *pcCodingParameter, UInt uiLayer );
  ErrVal initBuffer( const VUI* pcVui, UInt uiLayerIndex);
  ErrVal uninit();

  Void setInitialOutputDelay( UInt uiInitialOutputDelay )  { m_uiInitialOutputDelay = uiInitialOutputDelay;}

  Void setLayerBits( UInt uiBits ) { m_uiLayerBits = uiBits; }
  UInt getLayerBits()              { return m_uiLayerBits; }
protected:
  ErrVal xInitHrd (const HRD& rcHrd, const HRD::HrdParamType eHrdParamType);
  ErrVal xCreateBufferingSeiHrd( HRD::HrdParamType eHrdParamType, const HRD &rcHrd, SEI::BufferingPeriod* pcBPSei);
  ErrVal xCalculateTiming( HRD::HrdParamType eHrdParamType, UInt uiSize, Bool bIsIdr );

  Double  m_dFieldTimeDelta;
  Double  m_dOutputFrequency;
  UInt    m_uiOutputTicks;
  Double  m_dClockFrequency;
  Double  m_dActualOutTime;
  Double  m_dActualInTime;
  Double  m_dLastBPTime;
  UInt    m_uiInitialOutputDelay;

  StatBuf< DynBuf< TimingUnit >,2 >  m_aacTiming;
  const HRD* m_apcHrd[2];

  CodingParameter *m_pcCodingParameter;

  FILE *m_pfFileDebug;
  Bool m_bInitDone;
  UInt m_uiLayerBits;
};

// h264 namespace end
H264AVC_NAMESPACE_END

#endif // !defined(AFX_Scheduler_H__4242CFD4_A40A_4FCE_B740_60624D030E86__INCLUDED_)
