/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#include "H264AVCVideoIoLib.h"
#include "WriteYuvToFile.h"


WriteYuvToFile::WriteYuvToFile()
{
}

WriteYuvToFile::~WriteYuvToFile()
{
}

ErrVal
WriteYuvToFile::create( WriteYuvToFile*& rpcWriteYuv )
{
  rpcWriteYuv = new WriteYuvToFile;
  ROF( rpcWriteYuv )
  return Err::m_nOK;
}

ErrVal
WriteYuvToFile::destroy()
{
  ROT( m_cFile.is_open() );
  delete this;
  return Err::m_nOK;
}

ErrVal
WriteYuvToFile::init( const std::string& rcFileName )
{
  ROF( rcFileName.size() );
  if( Err::m_nOK != m_cFile.open( rcFileName, LargeFile::OM_WRITEONLY ) )
  {
    std::cerr << "failed to open YUV output file " << rcFileName.data() << std::endl;
    return Err::m_nERR;
  }
  return Err::m_nOK;
}

ErrVal
WriteYuvToFile::uninit()
{
  if( m_cFile.is_open() )
  {
    RNOK( m_cFile.close() );
  }
  return Err::m_nOK;
}

ErrVal
WriteYuvToFile::writeFrame( const UChar* pLum,
                            const UChar* pCb,
                            const UChar* pCr,
                            UInt         uiHeight,
                            UInt         uiWidth,
                            UInt         uiStride,
                            const UInt   rauiCropping[] )
{
  ROF( m_cFile.is_open() );

  UInt          y;
  const UChar*  pucSrc;

  uiWidth   -= rauiCropping[0] + rauiCropping[1];
  uiHeight  -= rauiCropping[2] + rauiCropping[3];

  pucSrc = pLum + ( rauiCropping[0] + rauiCropping[2] * uiStride );
  for( y = 0; y < uiHeight; y++ )
  {
    RNOK( m_cFile.write( pucSrc, uiWidth ) );
    pucSrc += uiStride;
  }

  uiStride >>= 1;
  uiHeight >>= 1;
  uiWidth  >>= 1;

  pucSrc = pCb + ( ( rauiCropping[0] + rauiCropping[2] * uiStride ) >> 1 );
  for( y = 0; y < uiHeight; y++ )
  {
    RNOK( m_cFile.write( pucSrc, uiWidth ) );
    pucSrc += uiStride;
  }

  pucSrc = pCr + ( ( rauiCropping[0] + rauiCropping[2] * uiStride ) >> 1 );
  for( y = 0; y < uiHeight; y++ )
  {
    RNOK( m_cFile.write( pucSrc, uiWidth ) );
    pucSrc += uiStride;
  }

  return Err::m_nOK;
}

