/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#include "H264AVCVideoIoLib.h"
#include "WriteYuvaToDisplay.h"
#include "WriteYuvaToRgb.h"
#include "WriteYuvaToRgbMMX.h"
#include "Display.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


#ifndef NO_MMX_SUPPORT
#include "CpuInfoIf.h"
#endif


ErrVal WriteYuvaToDisplay::xUseMMX()
{
#ifndef NO_MMX_SUPPORT

  CpuInfoIf* pcCpuInfo = NULL;

  RNOK( CpuInfoIf::create( pcCpuInfo ) );

  m_bMMX = (0 < pcCpuInfo->hasMmx());

  delete pcCpuInfo;
  pcCpuInfo = NULL;
#endif
  return Err::m_nOK;
}

WriteYuvaToDisplay::WriteYuvaToDisplay() :
  m_pcDisplay( NULL ),
  m_pcWriteYuvToRgb( NULL ),
  m_uiHeight( 0 ),
  m_uiWidth( 0 ),
  m_bInitDone( false ),
  m_bMMX( false )
{

}

WriteYuvaToDisplay::~WriteYuvaToDisplay()
{

}


ErrVal WriteYuvaToDisplay::create( WriteYuvIf*& rpcWriteYuv, const std::string& rcTitle )
{
  WriteYuvaToDisplay* pcWriteYuvaToDisplay;

  pcWriteYuvaToDisplay = new WriteYuvaToDisplay;

  rpcWriteYuv = pcWriteYuvaToDisplay;

  ROT( NULL == pcWriteYuvaToDisplay );

  pcWriteYuvaToDisplay->m_cTitle = rcTitle;

  RNOK( pcWriteYuvaToDisplay->xUseMMX() );
  return Err::m_nOK;
}


ErrVal WriteYuvaToDisplay::destroy()
{
  if( m_pcDisplay )
  {
    RNOK( m_pcDisplay->destroy() );
  }

  if( m_pcWriteYuvToRgb )
  {
    RNOK( m_pcWriteYuvToRgb->destroy() );
  }

  delete this;

  return Err::m_nOK;
}


ErrVal WriteYuvaToDisplay::xSetFrameDimension( UInt uiLumHeight, UInt uiLumWidth )
{
  //ROT( m_bInitDone );

  ROTRS( 0 == uiLumHeight || 0 ==  uiLumWidth, Err::m_nInvalidParameter );
  ROTRS( m_uiHeight == uiLumHeight && m_uiWidth ==  uiLumWidth, Err::m_nOK );

  if( m_pcDisplay )
  {
    RNOK( m_pcDisplay->destroy() );
  }

  if( m_pcWriteYuvToRgb )
  {
    RNOK( m_pcWriteYuvToRgb->destroy() );
  }

  m_uiHeight = uiLumHeight;
  m_uiWidth =  uiLumWidth;

  if( m_bMMX && 0 == (uiLumWidth & 7))
  {
    RNOK( WriteYuvaToRgbMMX::create( m_pcWriteYuvToRgb ) );
  }
  else
  {
    RNOK( WriteYuvaToRgb::create( m_pcWriteYuvToRgb ) );
  }

  RNOK( DisplayRGB::create( m_pcDisplay, m_uiWidth, m_uiHeight, m_cTitle ) );

  RNOK( m_pcWriteYuvToRgb->setFrameDimension( m_uiHeight, m_uiWidth ) );
  m_bInitDone = true;

  return Err::m_nOK;
}


ErrVal WriteYuvaToDisplay::writeFrame( const UChar *pLum,
                                       const UChar *pCb,
                                       const UChar *pCr,
                                       UInt uiLumHeight,
                                       UInt uiLumWidth,
                                       UInt uiLumStride )
{

  RNOK( xSetFrameDimension( uiLumHeight, uiLumWidth ) );

  UChar  *pucDisplayBuffer;
  UInt  uiBufferStride;

  RNOK( m_pcDisplay->getDisplayBuffer( pucDisplayBuffer, uiBufferStride ) );

  ROF( m_bInitDone );
  if( ! m_pcDisplay->isHardwareAccelerationInUse() )
  {
    RNOK( m_pcWriteYuvToRgb->writeFrameRGB( pucDisplayBuffer, uiBufferStride,
                                            pLum, pCb, pCr,
                                            uiLumHeight, uiLumWidth, uiLumStride ) );
  }
  else if( m_pcDisplay->isHardwareAccelerationInUse() == 0x32595559 )
  {
    RNOK( m_pcWriteYuvToRgb->writeFrameYUYV( pucDisplayBuffer, uiBufferStride,
                                             pLum, pCb, pCr,
                                             uiLumHeight, uiLumWidth, uiLumStride ) );
  }
  else if( m_pcDisplay->isHardwareAccelerationInUse() == 0x32315659 )
  {
    RNOK( m_pcWriteYuvToRgb->writeFrameYV12( pucDisplayBuffer, uiBufferStride,
                                             pLum, pCb, pCr,
                                             uiLumHeight, uiLumWidth, uiLumStride ) );
  }
  else
  {
    AOT(1);
  }

  RNOK( m_pcDisplay->displayFrame( pucDisplayBuffer ) );

  return Err::m_nOK;
}


ErrVal WriteYuvaToDisplay::writeField( Bool bTop,
                                       const UChar *pLum,
                                       const UChar *pCb,
                                       const UChar *pCr,
                                       UInt uiLumHeight,
                                       UInt uiLumWidth,
                                       UInt uiLumStride )
{

  RNOK( xSetFrameDimension( uiLumHeight*2, uiLumWidth ) );

  UChar  *pucDisplayBuffer;
  UInt  uiBufferStride;

  RNOK( m_pcDisplay->getDisplayBuffer( pucDisplayBuffer, uiBufferStride ) );

  if( ! bTop )
  {
    pucDisplayBuffer += uiBufferStride;
  }


  ROF( m_bInitDone );
  if( ! m_pcDisplay->isHardwareAccelerationInUse() )
  {
    RNOK( m_pcWriteYuvToRgb->writeFrameRGB( pucDisplayBuffer, uiBufferStride*2,
                                            pLum, pCb, pCr,
                                            uiLumHeight, uiLumWidth, uiLumStride ) );
  }
  else if( m_pcDisplay->isHardwareAccelerationInUse() == 0x32595559 )
  {
    RNOK( m_pcWriteYuvToRgb->writeFrameYUYV( pucDisplayBuffer, uiBufferStride*2,
                                             pLum, pCb, pCr,
                                             uiLumHeight, uiLumWidth, uiLumStride ) );
  }
  else if( m_pcDisplay->isHardwareAccelerationInUse() == 0x32315659 )
  {
    RNOK( m_pcWriteYuvToRgb->writeFrameYV12( pucDisplayBuffer, uiBufferStride*2,
                                             pLum, pCb, pCr,
                                             uiLumHeight, uiLumWidth, uiLumStride ) );
  }
  else
  {
    AOT(1);
  }

  RNOK( m_pcDisplay->displayFrame( pucDisplayBuffer ) );

  return Err::m_nOK;
}

