/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#ifndef _SCALABLE_MODIFY_CODE_
#define _SCALABLE_MODIFY_CODE_

#include "H264AVCCommonLib.h"
#include "H264AVCCommonLib/Sei.h"

class ScalableSEIModifyCode
{
protected:
	ScalableSEIModifyCode();
	~ScalableSEIModifyCode()	{}

public:
  static ErrVal Create         ( ScalableSEIModifyCode*& rpcScalableSEIModifyCode );
	ErrVal Destroy               ();
	ErrVal init                  ( ULong* pulStream, UInt uiBytes );
	ErrVal Uninit                ();
  ErrVal Write                 ( UInt uiBits, UInt uiNumberOfBits );
	ErrVal WriteUVLC             ( UInt uiValue );
	ErrVal WriteFlag             ( Bool bFlag );
	ErrVal WriteCode             ( UInt uiValue, UInt uiLength );
	ErrVal WriteAlignZero        ();
	ErrVal flushBuffer           ();
	UInt	 getNumberOfWrittenBits() { return m_uiBitsWritten; }
	ErrVal WritePayloadHeader    ( enum h264::SEI::MessageType eType, UInt uiSize );
	ErrVal WriteTrailingBits     ();
	ErrVal ConvertRBSPToPayload  ( UChar* m_pucBuffer, UChar pucStreamPacket[], UInt& uiBits, UInt uiHeaderBytes );

	ErrVal SEICode	( h264::SEI::ScalableSei* pcScalableSei, ScalableSEIModifyCode *pcScalableCodeIf );
	ErrVal SEICode	( h264::SEI::ScalableSeiLayersNotPresent* pcScalableSeiLayersNotPresent, ScalableSEIModifyCode *pcScalableCodeIf );
	ErrVal SEICode	( h264::SEI::ScalableSeiDependencyChange* pcScalableSeiDependencyChange, ScalableSEIModifyCode *pcScalableCodeIf );

protected:
	ULong  xSwap( ULong ul )
	{
		// heiko.schwarz@hhi.fhg.de: support for BSD systems as proposed by Steffen Kamp [kamp@ient.rwth-aachen.de]
#ifdef MSYS_BIG_ENDIAN
		return ul;
#else
		UInt ul2;

		ul2  = ul>>24;
		ul2 |= (ul>>8) & 0x0000ff00;
		ul2 |= (ul<<8) & 0x00ff0000;
		ul2 |= ul<<24;

		return ul2;
#endif
	}

private:
	BinData *m_pcBinData;
	ULong *m_pulStreamPacket;
	UInt m_uiBitCounter;
	UInt m_uiPosCounter;
	UInt m_uiDWordsLeft;
	UInt m_uiBitsWritten;
	UInt m_iValidBits;
	ULong m_ulCurrentBits;
	UInt m_uiCoeffCost;
};

#endif




