/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#include "H264AVCDecoderLibTest.h"
#include "H264AVCDecoderTest.h"


int
main( int argc, char** argv)
{
#ifdef SHARP_AVC_REWRITE_OUTPUT
  printf( "JSVM %s AVC REWRITER\n\n\n", _JSVM_VERSION_ );
#else
  printf( "JSVM %s Decoder\n\n\n",      _JSVM_VERSION_ );
#endif

  H264AVCDecoderTest*   pcH264AVCDecoderTest  = 0;
  ReadBitstreamFile*    pcReadStream          = 0;
#ifdef SHARP_AVC_REWRITE_OUTPUT
  WriteBitstreamToFile* pcWriteStream         = 0;
#else
  WriteYuvToFile*       pcWriteYuv            = 0;
#endif
  DecoderParameter      cParameter;

  //===== create instances =====
  RNOKRS( ReadBitstreamFile   ::create  ( pcReadStream ),                             -1 );
#ifdef SHARP_AVC_REWRITE_OUTPUT
  RNOKRS( WriteBitstreamToFile::create  ( pcWriteStream ),                            -2 );
#else
  RNOKRS( WriteYuvToFile      ::create  ( pcWriteYuv ),                               -2 );
#endif
  RNOKRS( H264AVCDecoderTest  ::create  ( pcH264AVCDecoderTest ),                     -3 );

  //===== initialization =====
  RNOKRS( cParameter           .init    ( argc, argv ),                               -4 );
  RNOKRS( pcReadStream        ->init    ( cParameter.cBitstreamFile ),                -5 );
#ifdef SHARP_AVC_REWRITE_OUTPUT
  RNOKRS( pcWriteStream       ->init    ( cParameter.cYuvFile ),                      -6 );
  RNOKRS( pcH264AVCDecoderTest->init    ( &cParameter, pcReadStream, pcWriteStream ), -7 );
#else
  RNOKRS( pcWriteYuv          ->init    ( cParameter.cYuvFile ),                      -6 );
  RNOKRS( pcH264AVCDecoderTest->init    ( &cParameter, pcReadStream, pcWriteYuv ),    -7 );
#endif

  //===== run =====
  RNOKR ( pcH264AVCDecoderTest->go      (),                                           -8 );

  //===== uninit and destroy instances =====
  RNOKR ( pcH264AVCDecoderTest->uninit  (),                                           -9 );
  RNOKR ( pcH264AVCDecoderTest->destroy (),                                           -10 );
  if( pcReadStream )
  {
    RNOKR ( pcReadStream      ->uninit  (),                                           -11 );
    RNOKR ( pcReadStream      ->destroy (),                                           -12 );
  }
#ifdef SHARP_AVC_REWRITE_OUTPUT
  if( pcWriteStream )
  {
    RNOKR ( pcWriteStream     ->uninit  (),                                           -13 );
    RNOKR ( pcWriteStream     ->destroy (),                                           -14 );
  }
#else
  if( pcWriteYuv )
  {
    RNOKR ( pcWriteYuv        ->uninit  (),                                           -13 );
    RNOKR ( pcWriteYuv        ->destroy (),                                           -14 );
  }
#endif

  return 0;
}
