/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#ifndef __H264AVCENCODERTEST_H_D65BE9B4_A8DA_11D3_AFE7_005004464B79
#define __H264AVCENCODERTEST_H_D65BE9B4_A8DA_11D3_AFE7_005004464B79

#include <algorithm>
#include <list>

#include "WriteBitstreamToFile.h"
#include "ReadYuvFile.h"
#include "WriteYuvToFile.h"



class EncoderCodingParameter;



typedef struct
{
  UInt    uiNumberOfLayers;
  std::string cBitstreamFilename;
  Int     nResult;
  UInt    nFrames;
} EncoderIoParameter;




class H264AVCEncoderTest
{
private:
  H264AVCEncoderTest();
  virtual ~H264AVCEncoderTest();

public:
  static ErrVal create( H264AVCEncoderTest*& rpcH264AVCEncoderTest );

  ErrVal init     ( Int     argc,
                    Char**  argv );
  ErrVal go       ();
  ErrVal destroy  ();
  ErrVal ScalableDealing ();

protected:
  ErrVal  xGetNewPicBuffer( PicBuffer*&             rpcPicBuffer,
                            UInt                    uiLayer,
                            UInt                    uiSize );
  ErrVal  xRemovePicBuffer( PicBufferList&          rcPicBufferUnusedList,
                            UInt                    uiLayer );

  ErrVal  xWrite          ( ExtBinDataAccessorList& rcList,
                            UInt&                   ruiBytesInFrame );
  ErrVal  xRelease        ( ExtBinDataAccessorList& rcList );

  ErrVal  xWrite          ( PicBufferList&          rcList,
                            UInt                    uiLayer );
  ErrVal  xRelease        ( PicBufferList&          rcList,
                            UInt                    uiLayer );

protected:
  EncoderIoParameter            m_cEncoderIoParameter;
  EncoderCodingParameter*       m_pcEncoderCodingParameter;
  h264::CreaterH264AVCEncoder*  m_pcH264AVCEncoder;
  WriteBitstreamToFile*         m_pcWriteBitstreamToFile;
  WriteYuvToFile*               m_apcWriteYuv           [MAX_LAYERS];
  ReadYuvFile*                  m_apcReadYuv            [MAX_LAYERS];

  PicBufferList                 m_acActivePicBufferList [MAX_LAYERS];
  PicBufferList                 m_acUnusedPicBufferList [MAX_LAYERS];
  UInt                          m_auiLumOffset          [MAX_LAYERS];
  UInt                          m_auiCbOffset           [MAX_LAYERS];
  UInt                          m_auiCrOffset           [MAX_LAYERS];
  UInt                          m_auiHeight             [MAX_LAYERS];
  UInt                          m_auiWidth              [MAX_LAYERS];
  UInt                          m_auiStride             [MAX_LAYERS];
  UInt                          m_aauiCropping          [MAX_LAYERS][4];

  UChar                         m_aucStartCodeBuffer[5];
  BinData                       m_cBinDataStartCode;
  std::string                   m_cWriteToBitFileName;
  std::string                   m_cWriteToBitFileTempName;
};




#endif //__H264AVCENCODERTEST_H_D65BE9B4_A8DA_11D3_AFE7_005004464B79
