/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#if !defined  _MCTF_H_
#define       _MCTF_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


class PreProcessorParameter;

H264AVC_NAMESPACE_BEGIN

class SliceHeader;
class MbDataCtrl;
class MbEncoder;
class MotionEstimation;
class Frame;


class MCTF
{
protected:
  MCTF          ();
  virtual ~MCTF ();

public:
  static ErrVal create              ( MCTF*&                      rpcMCTF );
  ErrVal        destroy             ();
  ErrVal        init                ( PreProcessorParameter*      pcParameter,
                                      MbEncoder*                  pcMbEncoder,
                                      YuvBufferCtrl*              pcYuvFullPelBufferCtrl,
                                      YuvBufferCtrl*              pcYuvHalfPelBufferCtrl,
                                      QuarterPelFilter*           pcQuarterPelFilter,
                                      MotionEstimation*           pcMotionEstimation );
  ErrVal        uninit              ();
  ErrVal        process             ( PicBuffer*                  pcOrgPicBuffer,
                                      PicBuffer*                  pcRecPicBuffer,
                                      PicBufferList&              rcPicBufferOutputList,
                                      PicBufferList&              rcPicBufferUnusedList );

protected:
  //===== main functions =====
  ErrVal        xProcessGOP         ( PicBufferList&              rcPicBufferInputList,
                                      PicBufferList&              rcPicBufferOutputList,
                                      PicBufferList&              rcPicBufferUnusedList );

  //===== data management =====
  ErrVal  xCreateData               ( const SequenceParameterSet& rcSPS );
  ErrVal  xDeleteData               ();
  ErrVal  xInitGOP                  ( PicBufferList&              rcPicBufferInputList );
  ErrVal  xFinishGOP                ( PicBufferList&              rcPicBufferInputList,
                                      PicBufferList&              rcPicBufferOutputList,
                                      PicBufferList&              rcPicBufferUnusedList );
  ErrVal  xStoreReconstruction      ( PicBufferList&              rcPicBufferOutputList );

  //===== decomposition / composition =====
  ErrVal  xMotionEstimationStage    ( UInt                        uiBaseLevel );
  ErrVal  xDecompositionStage       ( UInt                        uiBaseLevel );
  ErrVal  xCompositionStage         ( UInt                        uiBaseLevel,
                                      PicBufferList&              rcPicBufferInputList );

  //===== control data initialization =====
  ErrVal  xSetScalingFactors        ( UInt                        uiBaseLevel );
  ErrVal  xGetListSizes             ( UInt                        uiTemporalLevel,
                                      UInt                        uiFrameIdInGOP,
                                      UInt                        auiPredListSize[2],
                                      UInt                        aauiUpdListSize[MAX_DSTAGES][2] );
  ErrVal  xInitSliceHeader          ( UInt                        uiTemporalLevel,
                                      UInt                        uiFrameIdInGOP );
  ErrVal  xClearBufferExtensions    ();
  ErrVal  xGetPredictionLists       ( RefFrameList&               rcRefList0,
                                      RefFrameList&               rcRefList1,
                                      UInt                        uiBaseLevel,
                                      UInt                        uiFrame,
                                      Bool                        bHalfPel );
  ErrVal  xGetUpdateLists           ( RefFrameList&               rcRefList0,
                                      RefFrameList&               rcRefList1,
                                      CtrlDataList&               rcCtrlList0,
                                      CtrlDataList&               rcCtrlList1,
                                      UInt                        uiBaseLevel,
                                      UInt                        uiFrame );
  ErrVal  xInitControlDataMotion    ( UInt                        uiBaseLevel,
                                      UInt                        uiFrame,
                                      Bool                        bMotionEstimation );

  //===== motion estimation / compensation =====
  ErrVal  xMotionCompensation       ( Frame*                   pcMCFrame,
                                      RefFrameList*               pcRefFrameList0,
                                      RefFrameList*               pcRefFrameList1,
                                      MbDataCtrl*                 pcMbDataCtrl,
                                      SliceHeader&                rcSH );
  ErrVal  xMotionEstimation         ( RefFrameList*               pcRefFrameList0,
                                      RefFrameList*               pcRefFrameList1,
                                      const Frame*             pcOrigFrame,
                                      ControlData&                rcControlData );
  ErrVal  xUpdateCompensation       ( Frame*                   pcMCFrame,
                                      RefFrameList*               pcRefFrameList,
                                      CtrlDataList*               pcCtrlDataList,
                                      ListIdx                     eListUpd );

  //===== auxiliary functions =====
  ErrVal  xFillAndUpsampleFrame     ( Frame*                   rcFrame );
  ErrVal  xFillAndExtendFrame       ( Frame*                   rcFrame );
  ErrVal  xZeroIntraMacroblocks     ( Frame*                   pcFrame,
                                      ControlData&                pcCtrlData );

protected:
  SequenceParameterSet*         m_pcSPS;
  PictureParameterSet*          m_pcPPS;
  YuvBufferCtrl*                m_pcYuvFullPelBufferCtrl;
  YuvBufferCtrl*                m_pcYuvHalfPelBufferCtrl;
  MbEncoder*                    m_pcMbEncoder;
  QuarterPelFilter*             m_pcQuarterPelFilter;
  MotionEstimation*             m_pcMotionEstimation;

  Bool                          m_bFirstGOPCoded;                     // true if first GOP of a sequence has been coded
  UInt                          m_uiGOPSize;                          // current GOP size
  UInt                          m_uiDecompositionStages;              // number of decomposition stages
  UInt                          m_uiFrameWidthInMb;                   // frame width in macroblocks
  UInt                          m_uiFrameHeightInMb;                  // frame height in macroblocks
  UInt                          m_uiMbNumber;                         // number of macroblocks in a frame
  Double                        m_adBaseQpLambdaMotion[MAX_DSTAGES];  // base QP's for mode decision and motion estimation
  Frame*                        m_pcFrameTemp;                        // auxiliary frame memory
  Frame**                       m_papcFrame;                          // frame stores
  Frame**                       m_papcResidual;                       // frame stores for residual data
  ControlData*                  m_pacControlData;                     // control data arrays
  PicBufferList                 m_cOrgPicBufferList;
  PicBufferList                 m_cRecPicBufferList;
};

H264AVC_NAMESPACE_END

#endif // _MCTF_H_
