/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#if !defined  _MCTF_PRE_PROCESSOR_H_
#define       _MCTF_PRE_PROCESSOR_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


class PreProcessorParameter;

H264AVC_NAMESPACE_BEGIN

class Transform;
class YuvBufferCtrl;
class QuarterPelFilter;
class SampleWeighting;
class MbEncoder;
class IntraPredictionSearch;
class MotionEstimation;
class MotionEstimationQuarterPel;
class RateDistortion;
class MCTF;
class XDistortion;
class CodingParameter;


class MCTFPreProcessor
{
protected:
  MCTFPreProcessor          ();
  virtual ~MCTFPreProcessor ();

public:
  static ErrVal create      ( MCTFPreProcessor*&      rpcMCTFPreProcessor );
  ErrVal        destroy     ();

  ErrVal init               ( PreProcessorParameter*  pcParameter,
                              CodingParameter*        pcCodingParameter );
  ErrVal uninit             ();
  ErrVal process            ( PicBuffer*              pcOriginalPicBuffer,
                              PicBuffer*              pcReconstructPicBuffer,
                              PicBufferList&          rcPicBufferOutputList,
                              PicBufferList&          rcPicBufferUnusedList );
  ErrVal finish             ( PicBufferList&          rcPicBufferOutputList,
                              PicBufferList&          rcPicBufferUnusedList );

protected:
  ErrVal xCreateMCTFPreProcessor();

protected:
  MCTF*                   m_pcMCTF;
  MbEncoder*              m_pcMbEncoder;
  Transform*              m_pcTransform;
  IntraPredictionSearch*  m_pcIntraPrediction;
  YuvBufferCtrl*          m_pcYuvFullPelBufferCtrl;
  YuvBufferCtrl*          m_pcYuvHalfPelBufferCtrl;
  QuarterPelFilter*       m_pcQuarterPelFilter;
  SampleWeighting*        m_pcSampleWeighting;
  XDistortion*            m_pcXDistortion;
  MotionEstimation*       m_pcMotionEstimation;
  RateDistortion*         m_pcRateDistortion;
};


H264AVC_NAMESPACE_END


#endif // _MCTF_PRE_PROCESSOR_H_

