/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#ifndef _MCTF_PRE_PROCESSOR_TEST_H
#define _MCTF_PRE_PROCESSOR_TEST_H


#include "Typedefs.h"
#include "H264AVCEncoderLib.h"
#include "H264AVCVideoIoLib.h"
#include "MCTFPreProcessor.h"
#include "PreProcessorParameter.h"
#include "WriteYuvToFile.h"
#include "ReadYuvFile.h"
#include "CodingParameter.h"


class MCTFPreProcessorTest
{
public:
  MCTFPreProcessorTest    ();
  ~MCTFPreProcessorTest   ();

  static ErrVal   create  ( MCTFPreProcessorTest*&  rpcMCTFPreProcessorTest );
  ErrVal          destroy ();
  ErrVal          init    ( PreProcessorParameter*  pcPreProcessorParameter );
  ErrVal          go      ();

protected:
  ErrVal  xInitCodingParameter  ();
  ErrVal  xGetNewPicBuffer      ( PicBuffer*&     rpcPicBuffer,
                                  UInt            uiSize );
  ErrVal  xRemovePicBuffer      ( PicBufferList&  rcList );
  ErrVal  xWrite                ( PicBufferList&  rcList );

private:
  PreProcessorParameter*        m_pcParameter;
  h264::CodingParameter         m_cCodingParameter;
  h264::MCTFPreProcessor*       m_pcMCTFPreProcessor;
  WriteYuvToFile*               m_pcWriteYuv;
  ReadYuvFile*                  m_pcReadYuv;
  PicBufferList                 m_cActivePicBufferList;
  PicBufferList                 m_cUnusedPicBufferList;
  UInt                          m_uiPicSize;
  UInt                          m_uiLumOffset;
  UInt                          m_uiCbOffset;
  UInt                          m_uiCrOffset;
  UInt                          m_uiHeight;
  UInt                          m_uiWidth;
  UInt                          m_uiStride;
};


#endif // _MCTF_PRE_PROCESSOR_TEST_H

