/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#ifndef _PRE_PROCESSOR_PARAMETER_H_
#define _PRE_PROCESSOR_PARAMETER_H_

#include "Typedefs.h"
#include "string.h"
#include "H264AVCCommonLib.h"



class PreProcessorParameter
{
public:
  PreProcessorParameter();
  ~PreProcessorParameter();

  static ErrVal create( PreProcessorParameter*& rpcPreProcessorParameter );
  ErrVal        destroy();
  ErrVal        init  ( Int argc, Char** argv );

  const std::string&  getInputFileName   () const { return m_cInputFileName;  }
  const std::string&  getOutputFileName  () const { return m_cOutputFileName;  }
  UInt                getFrameWidth      () const { return m_uiFrameWidth; }
  UInt                getFrameHeight     () const { return m_uiFrameHeight; }
  UInt                getNumFrames       () const { return m_uiNumFrames; }
  UInt                getGOPSize         () const { return m_uiGOPSize; }
  Double              getQP              () const { return m_dQP; }

protected:
  ErrVal              xPrintUsage        ( Char** argv );

private:
  std::string   m_cInputFileName;
  std::string   m_cOutputFileName;
  UInt          m_uiFrameWidth;
  UInt          m_uiFrameHeight;
  UInt          m_uiNumFrames;
  UInt          m_uiGOPSize;
  Double        m_dQP;
};


#endif //_PRE_PROCESSOR_PARAMETER_H_

