/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#ifndef _QUALITY_LEVEL_ESTIMATION_H
#define _QUALITY_LEVEL_ESTIMATION_H


#include "Typedefs.h"
#include "H264AVCCommonLib.h"




class FGSPacketEntry;
class QualityLayer;
typedef MyList<QualityLayer>    QualityLayerList;
typedef MyList<FGSPacketEntry*> FGSPacketList;




class FGSPacketEntry
{
  friend class QualityLayer;

protected:
  static const Double forbiddenDist;

public:
  FGSPacketEntry ();
  ~FGSPacketEntry();

  Void    setQualityLevel ( UInt    uiQualityLevel );
  ErrVal  init            ( UInt    uiLayer,
                            UInt    uiFrameID,
                            UInt    uiFGSLayer,
                            UInt    uiPacketRate,
                            Double  dDeltaDistortion );
  ErrVal  uninit          ();

  Bool    isValid         () const;
  UInt    getQualityId () const  { return m_uiQualityId; }
  UInt    getFGSLayer     () const  { return m_uiFGSLayer; }
  UInt    getFrameID      () const  { return m_uiFrameID; }
  UInt    getLayer        () const  { return m_uiLayer; };

protected:
  UInt    m_uiFrameID;
  UInt    m_uiFGSLayer;
  UInt    m_uiRate;
  Double  m_dDeltaDistortion;
  UInt    m_uiQualityId;
  //JVT-S043
  UInt    m_uiLayer;
};





class QualityLayer
{
public:
  static const Double maxCost;

public:
  QualityLayer  ( const QualityLayer&   rcQualityLayer   );
  QualityLayer  ( const FGSPacketEntry& rcFGSPacketEntry );
  QualityLayer  ( FGSPacketList&        rcFGSPacketList  );
  ~QualityLayer ();

  FGSPacketList&  getFGSPacketList  ();
  Void            add               ( FGSPacketList&      rcFGSPacketList );
  Void            remove            ( FGSPacketList&      rcFGSPacketList );
  Double          getSeperateArea   ( const QualityLayer& rcQualityLayer );
  Double          getCombinedArea   ( const QualityLayer& rcQualityLayer );
  Void            merge             ( const QualityLayer& rcQualityLayer );

  Void            conditionedMerge  (       QualityLayer& rcQualityLayer );
  Bool            isMergingPossible (       QualityLayer& rcQualityLayer,
                                            Double&       dMergeCost );

public:
  Double  getSlope    ()                          const { return m_dSlope; }
  Bool    operator >  ( const QualityLayer& ql1 ) const { return ql1.getSlope() < getSlope(); } // operator for sorting

protected:
  UInt          m_uiRate;
  Double        m_dDeltaDistortion;
  Double        m_dSlope;
  FGSPacketList m_cFGSPacketList;
};




class QualityLevelEstimation
{
public:
  QualityLevelEstimation  ();
  ~QualityLevelEstimation ();

  ErrVal  init                  ( UInt    uiNumLayers,
                                  UInt    pauiNumFGSLayers[],
                                  UInt    pauiNumFrames[] );
  ErrVal  uninit                ();

  ErrVal  addPacket             ( UInt    uiLayer,
                                  UInt    uiFGSLayer,
                                  UInt    uiFrameNumInCodingOrder,
                                  UInt    uiPacketSize,
                                  Double  dDeltaDistortion );

  ErrVal  optimizeQualityLevel  ( UInt    uiTopLayer,
                                  UInt    uiMinLayer,
                                  UInt    uiMinLevel,
                                  UInt    uiMaxLevel );

  UInt    getQualityId       ( UInt    uiLayer,
                                  UInt    uiFGSLayer,
                                  UInt    uiFrameNumInCodingOrder ) const;

private:
  UInt              m_uiNumLayers;
  UInt              m_auiNumFGSPackets[MAX_LAYERS];
  UInt              m_auiNumFrames[MAX_LAYERS];
  FGSPacketEntry*   m_aaacFGSPacketEntry[MAX_LAYERS][MAX_QUALITY_LEVELS];
};



#endif // _QUALITY_LEVEL_ESTIMATION_H

