/*
********************************************************************************

This software module was originally developed by

Heiko Schwarz    (Fraunhofer HHI),
Tobias Hinz      (Fraunhofer HHI),
Karsten Suehring (Fraunhofer HHI)

in the course of development of the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video
Coding) for reference purposes and its performance may not have been optimized.
This software module is an implementation of one or more tools as specified by
the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding).

Those intending to use this software module in products are advised that its
use may infringe existing patents. ISO/IEC have no liability for use of this
software module or modifications thereof.

Assurance that the originally developed software module can be used
(1) in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) once the
ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) has been adopted; and
(2) to develop the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding): 

To the extent that Fraunhofer HHI owns patent rights that would be required to
make, use, or sell the originally developed software module or portions thereof
included in the ISO/IEC 14496-10:2005 Amd.1 (Scalable Video Coding) in a
conforming product, Fraunhofer HHI will assure the ISO/IEC that it is willing
to negotiate licenses under reasonable and non-discriminatory terms and
conditions with applicants throughout the world.

Fraunhofer HHI retains full right to modify and use the code for its own
purpose, assign or donate the code to a third party and to inhibit third
parties from using the code for products that do not conform to MPEG-related
ITU Recommendations and/or ISO/IEC International Standards. 

This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 2005. 

********************************************************************************
*/


#ifndef _QUALITY_LEVEL_PARAMETER_H_
#define _QUALITY_LEVEL_PARAMETER_H_

#include "Typedefs.h"
#include "string.h"
#include "H264AVCCommonLib.h"

//JVT-S043
enum QLAssignerMode
{
  QLASSIGNERMODE_QL=0,
  QLASSIGNERMODE_MLQL=1
};

class QualityLevelParameter
{
public:
  QualityLevelParameter();
  ~QualityLevelParameter();

  static ErrVal create( QualityLevelParameter*& rpcQualityLevelParameter );
  //manu.mathew@samsung : memory leak fix
  ErrVal        destroy();
  //--
  ErrVal        init  ( Int argc, Char** argv );

  const std::string&  getInputBitStreamName   ()          const { return m_cInputBitStreamName;  }
  const std::string&  getOutputBitStreamName  ()          const { return m_cOutputBitStreamName;  }
  const std::string&  getOriginalFileName     ( UInt ui ) const { return m_acOriginalFileName[ui]; }
  const std::string&  getDataFileName         ()          const { return m_cDataFileName; }
  Bool                writeDataFile           ()          const { return m_uiDataFileMode == 2; }
  Bool                readDataFile            ()          const { return m_uiDataFileMode == 1; }
  Bool                useIndependentDistCalc  ()          const { return ( m_uiDistortionEstimationMode & 1) == 1; }
  Bool                useDependentDistCalc    ()          const { return ( m_uiDistortionEstimationMode & 2) == 2; }
  //Bool                writeQualityLayerSEI    ()          const { return m_bQualityLayerSEI; }//SEI changes update
  Bool                writePriorityLevelSEI    ()          const { return m_bPriorityLevelSEI; }//SEI changes update
  QLAssignerMode      getQLAssignerMode       ()          const { return m_eQLAssignerMode; }
protected:
  ErrVal              xPrintUsage             ( Char** argv );


private:
  std::string   m_cInputBitStreamName;
  std::string   m_cOutputBitStreamName;
  std::string   m_acOriginalFileName[MAX_LAYERS];
  std::string   m_cDataFileName;
  UInt          m_uiDataFileMode;
  UInt          m_uiDistortionEstimationMode;
  //Bool          m_bQualityLayerSEI;//SEI changes update
	Bool          m_bPriorityLevelSEI;//SEI changes update
  //JVT-S043
  QLAssignerMode  m_eQLAssignerMode;
};




#endif //_QUALITY_LEVEL_PARAMETER_H_

